import cPickle
import json
from django.db import models
from .serializers import SortedJSONEncoder


class CerialField(models.TextField):
    def loads(self, s):
        """
        Deserialize s
        """
        raise NotImplemented

    def dumps(self, obj):
        """
        Serialize obj
        """
        raise NotImplemented

    def pre_save(self, obj, create):
        value = obj.__dict__[self.name]
        if not isinstance(value, basestring):
            value = self.dumps(value)
        return value

    def contribute_to_class(self, cls, name):
        super(CerialField, self).contribute_to_class(cls, name)
        setattr(cls, self.name, CerialDescriptor(self))

    def south_field_triple(self):
        from south.modelsinspector import introspector
        args, kwargs = introspector(self)
        return 'django.db.models.TextField', args, kwargs


class CerialDescriptor(object):
    def __init__(self, field):
        self.field = field

    def __get__(self, obj, owner):
        value = obj.__dict__[self.field.name]
        # we don't try to deserialize empty strings
        if value and isinstance(value, basestring):
            value = self.field.loads(value)
            obj.__dict__[self.field.name] = value
        return value

    def __set__(self, obj, value):
        obj.__dict__[self.field.name] = value


class JSONField(CerialField):
    def loads(self, s):
        return json.loads(s)

    def dumps(self, obj):
        return json.dumps(obj, cls=SortedJSONEncoder)


class PickleField(CerialField):
    def loads(self, s):
        return cPickle.loads(str(s).decode('base64'))

    def dumps(self, obj):
        return cPickle.dumps(obj).encode('base64')




from django.core.serializers.json import DjangoJSONEncoder


class SortedJSONEncoder(DjangoJSONEncoder):
    """
    A json encoder that sorts the dict keys and that can encode datetime
    objects.
    """
    def __init__(self, **kwargs):
        kwargs['sort_keys'] = True
        super(SortedJSONEncoder, self).__init__(**kwargs)




from django.db import models
from cerial import JSONField, PickleField


class JSONEntry(models.Model):
    f = JSONField(blank=True)

class PickleEntry(models.Model):
    f = PickleField(blank=True)




#coding=utf-8
import datetime
from django.utils import unittest
from testapp.models import JSONEntry, PickleEntry


class A(object):
    def f(self, x):
        return x * x


class JSONTest(unittest.TestCase):
    def setUp(self):
        self.model = JSONEntry
        self.manager = self.model.objects

    def testCreate(self):
        entry = self.manager.create(pk=100, f={'k': 1})
        self.assertEqual(entry.f, {'k': 1})

    def testAssign(self):
        entry = self.manager.create(f={'k': 2})
        entry.f['k'] = {'j': 1}
        entry.save()
        self.assertEqual(entry.f, {'k': {'j': 1}})

    def testNULL(self):
        entry = self.manager.create(f=None)
        entry = self.manager.get(pk=entry.pk)
        self.assertEqual(entry.f, None)

    def testEmpty(self):
        entry = self.manager.create(f='')
        entry = self.manager.get(pk=entry.pk)
        self.assertEqual(entry.f, '')

    def testEmptyDict(self):
        entry = self.manager.create(f={})
        entry = self.manager.get(pk=entry.pk)
        self.assertEqual(entry.f, {})

    def testDate(self):
        d = datetime.date.today()
        entry = self.manager.create(f={'d': d})

    def testUnicode(self):
        entry = self.manager.create(f={'k': u'åäö'})
        entry = self.manager.get(pk=entry.pk)
        self.assertEqual(entry.f, {'k': u'åäö'})


class PickleTest(JSONTest):
    def setUp(self):
        self.model = PickleEntry
        self.manager = self.model.objects

    def testSimpleClass(self):
        entry = self.manager.create(f=A)
        entry = self.manager.get(pk=entry.pk)
        self.assertEqual(entry.f, A)




from setuptools import setup, find_packages


setup(
    name='django-cerial',
    version='0.0.2',
    description='',
    long_description=open('README.rst').read(),
    author='Mikko Hellsing',
    author_email='mikko@aino.se',
    license='BSD',
    url='https://github.com/aino/django-cerial',
    packages=find_packages(),
    zip_safe=False,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Framework :: Django',
    ],
)

